from PyQt4.QtGui import QDialog, QVBoxLayout
from PyQt4.QtCore import Qt

from tree_widget import ObjectTreeWidget


class ObjectTreeDialog(QDialog):

    def __init__(self, root_object, parent=None, f=Qt.WindowFlags()):
        QDialog.__init__(self, parent, f)
        self.resize(480, 640)
        self.vertical_layout = QVBoxLayout(self)
        self.tree_view = ObjectTreeWidget(self)
        self.tree_view.set_root_object(root_object)
        self.tree_view.expand_top_level()
        self.vertical_layout.addWidget(self.tree_view)


if __name__ == "__main__":
    from PyQt4.QtGui import QApplication
    app = QApplication([])
    dlg = ObjectTreeDialog([1, 2, 3, dict(a=3, b=4, c=(1, 2, dict(d=(4, 5))))])
    dlg.show()
    app.exec_()



from PyQt4.QtCore import QVariant, QAbstractItemModel, QModelIndex, Qt
from PyQt4.QtGui import QTreeView

from tree import Leaf, Node


class _TreeModel(QAbstractItemModel):

    def __init__(self, root_object, parent=None):
        super(_TreeModel, self).__init__(parent)
        self.root_node = Node.from_("root", root_object)

    def top_level_indices(self):
        for i in range(self.root_node.get_num_children()):
            idx = self.index(i, 0)
            if idx.isValid():
                yield idx

    def data(self, index, role):
        if not index.isValid():
            return QVariant()
        if role != Qt.DisplayRole:
            return QVariant()

        item = index.internalPointer()
        if index.column() == 0:
            return item.name
        else:
            if isinstance(item, Leaf):
                return str(item.value)
            return ""

    def flags(self, index):
        if not index.isValid():
            return 0
        return Qt.ItemIsEnabled | Qt.ItemIsSelectable

    def index(self, row, column, parent=QModelIndex()):
        if not self.hasIndex(row, column, parent):
            return QModelIndex()
        if not parent.isValid():
            child_item = self.root_node.children[row]
        else:
            parent_item = parent.internalPointer()
            child_item = parent_item.children[row]

        return self.createIndex(row, column, child_item)

    def parent(self, index):
        if not index.isValid():
            return QModelIndex()

        child_item = index.internalPointer()
        parent_item = child_item.get_parent()
        if parent_item == self.root_node:
            return QModelIndex()
        return self.createIndex(parent_item.get_index(), 0, parent_item)

    def rowCount(self, parent=QModelIndex()):
        if parent.column() > 0:
            return 0
        if not parent.isValid():
            parent_item = self.root_node
        else:
            parent_item = parent.internalPointer()
        return parent_item.get_num_children()

    def columnCount(self, parent=QModelIndex()):
        return 2

    def headerData(self, section, orientation, role):
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            return ["Node", "Value"][section]
        return QVariant()


class ObjectTreeWidget(QTreeView):

    def __init__(self, *a, **kw):
        super(ObjectTreeWidget, self).__init__(*a, **kw)
        self.setAlternatingRowColors(True)
        self.setAutoExpandDelay(0)
        self.setIndentation(20)
        self.setRootIsDecorated(True)
        self.setUniformRowHeights(True)
        self.setAnimated(True)
        self.setHeaderHidden(False)

    def set_root_object(self, root_object):
        self.root_object = root_object
        self.update()

    def update(self):
        self.setModel(None)
        self.setModel(_TreeModel(self.root_object))

    def expand_top_level(self):
        for idx in self.model().top_level_indices():
            self.setExpanded(idx, True)



from setuptools import setup, find_packages

description = "qt tree model + tree viewer for nested objects"

long_description = """
This Python package provides a Qt based widget and a dialog class for exploring a
nested object structure.

For examples see: http://github.com/uweschmitt/qt_object_viewer.git
"""

setup(name="qt_object_viewer",
      description=description,
      long_description=long_description,
      maintainer="Uwe Schmitt",
      maintainer_email="uwe.schmitt@id.ethz.ch",
      platforms=["any"],
      version="1.0.3",
      packages=find_packages(exclude=["tests"]),
      include_package_data=True,
      zip_safe=False,
      )



import re
import sys
import types


def type_to_str(t):
    match = re.match("<type '(.*)'>", str(t))
    if match is None:
        match = re.match("<class '(.*)'>", str(t))
        if match is None:
            return str(t)
    return match.groups()[0]


_do_not_recurse_into = (types.MethodType, types.BuiltinMethodType, types.UnboundMethodType,
                        types.MemberDescriptorType, types.ClassType, types.TypeType)


def interesting_attributes(obj):
    for name in dir(obj):
        value = getattr(obj, name)
        if name.startswith("_"):
            continue
        if isinstance(value, _do_not_recurse_into):
            continue
        yield name, value


class Node(object):

    def __init__(self, parent, type_, name):
        self.parent = parent
        self.type_ = type_
        self.name = name
        self.index = -1
        self.children = []

    def set_children(self, children):
        self.children = children
        for i, child in enumerate(children):
            child.index = i

    def get_children(self):
        return self.children

    def get_num_children(self):
        return len(self.children)

    def get_name(self):
        return self.name

    def get_index(self):
        return self.index

    def get_parent(self):
        return self.parent

    @classmethod
    def from_(clz, name, obj, parent=None):
        type_as_str = type_to_str(type(obj))
        if isinstance(obj, (int, float, basestring)):
            return Leaf(parent, type_as_str, name, obj)
        else:
            if isinstance(obj, _do_not_recurse_into):
                return Leaf(parent, type_as_str, name, obj)
            result = clz(parent, type_as_str, name)
            children = []
            if isinstance(obj, (list, tuple)):
                children = [clz.from_("", item, parent=result) for i, item in enumerate(obj)]
            elif isinstance(obj, set):
                children = [clz.from_("", item, parent=result) for item in obj]
            elif isinstance(obj, dict):
                children = [clz.from_(key, value, parent=result) for key, value in obj.items()]
            else:
                children = [clz.from_(key, value, parent=result) for key, value in interesting_attributes(obj)]
                if not children:
                    return Leaf(parent, type_as_str, name, obj)
            result.set_children(children)
            return result

    def print_(self, indent=0):
        sys.stdout.write(" " * indent)
        print "at %d: %s(%s)" % (self.index, self.name, self.type_)
        for child in self.children:
            child.print_(indent + 4)


class Leaf(Node):

    def __init__(self, parent, type_, name, value):
        super(Leaf, self).__init__(parent, type_, name)
        self.value = value

    def get_children(self):
        return None

    def get_num_children(self):
        return 0

    def print_(self, indent=0):
        sys.stdout.write(" " * indent)
        print "at %d: %s(%s) = %r" % (self.index, self.name, self.type_, self.value)
